import express from "express";
import CommonUtil from "../utils/commonUtil.js";
import {BigNumber, Contract, ethers, utils} from "ethers";
import L1GatewayRouterAbi from './contract/l1GatewayRouter.js'
import L2GatewayRouterAbi from './contract/l2GatewayRouter.js'
import mathUtil from "../utils/mathUtil.js";

const router = express.Router();
const l1_gateway_router = "0x13FBE0D0e5552b8c9c4AE9e2435F38f37355998a";
const l2_gateway_router = "0x9aD3c5617eCAa556d6E166787A97081907171230";
const scroll_network = "https://sepolia-rpc.scroll.io/";
const sepolia_network = "https://eth-sepolia.g.alchemy.com/v2/QR461hv0kEkKn91VlTAlIx6RhnnfPl-m";

router.post("/", async (req, res) => {
    let {wallet, params} = CommonUtil.getParamsAndWallet(req);
    let amountNumber = mathUtil.getRandomNumber(params.minAmount, params.maxAmount, params.accuracy);
    amountNumber = utils.parseEther(`${amountNumber}`);
    const providerSep = ethers.getDefaultProvider(sepolia_network);
    const providerScroll = ethers.getDefaultProvider(scroll_network);

    // const router = new Contract(rt_addr, RouterAbi, signer);
    let tx;
    let provider;
    let signer;
    if (params.distChain === "scroll") {
        signer = new ethers.Wallet(wallet.privateKey, providerSep);
        const l1GatewayRouter = new Contract(l1_gateway_router, L1GatewayRouterAbi, signer);
        const gasPrice = (await providerScroll.getGasPrice()).toNumber();
        tx = await l1GatewayRouter['depositETH(uint256,uint256)'](amountNumber, 168000, {
            value:amountNumber.add(BigNumber.from(168000).mul(gasPrice))
        });
        provider = providerSep;
    } else if (params.distChain === "sepolia") {
        signer = new ethers.Wallet(wallet.privateKey, providerScroll);
        const l2GatewayRouter = new Contract(l2_gateway_router, L2GatewayRouterAbi, signer);
        tx = await l2GatewayRouter['withdrawETH(uint256,uint256)'](amountNumber, 0, {
            value: amountNumber
        });
        provider = providerScroll;
    }
    await tx.wait();
    const txReceipt = await provider.getTransactionReceipt(tx.hash);
    tx = await provider.getTransaction(tx.hash);
    const gas = utils.formatEther((txReceipt.gasUsed * tx.gasPrice).toFixed(0));
    const balance = await signer.getBalance()
    const balanceStr = utils.formatEther(balance);
    const value = utils.formatEther(tx.value)
    res.send({
        code: 0,
        msg: "",
        data: {
            txId: tx.hash,
            gas: gas,
            currentBalance: balanceStr,
            status: txReceipt.status,
            value: value
        }
    });
});

export default router;